/*
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements. See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership. The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the  "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
#if !defined(XALANDUMMYIDENTWRITER_HEADER_GUARD_1357924680)
#define XALANDUMMYIDENTWRITER_HEADER_GUARD_1357924680


// Base include file.  Must be first.
#include <xalanc/XMLSupport/XMLSupportDefinitions.hpp>

#include <xalanc/Include/XalanVector.hpp>


namespace XALAN_CPP_NAMESPACE {


template<class WriteHandeler>
class XalanDummyIndentWriter
{
public:
    typedef typename WriteHandeler::size_type    size_type;
    typedef typename WriteHandeler::value_type   value_type;

    XalanDummyIndentWriter(
            WriteHandeler&  /*writer*/,
            size_type       /*ident*/)
    {
    }

    ~XalanDummyIndentWriter()
    {
    }
        
        
    void
    setIndent(size_type /*value*/)
    {
    }
    
    size_type
    getIndent()const
    {
        return 0;
    }
    
    void
    indent()
    {
    }
    
    void
    increaseIndent()
    {
    }
    
    void
    decreaseIndent()
    {
    }       
    
    void
    setStartNewLine(bool /*value*/)
    {
    }



    /**
     * Output a line break.
     */
    void
    outputLineSep()
    {
    }

    void
    setPrevText(bool /*value*/)
    {

    }

    void
    setPreserve(bool /*value*/)
    {
        // Empty of the function
    }

    void
    pop_preserve()
    {
        // Empty body of the function
    }

    void
    push_preserve()
    {
        // Empty of the function
    }

private:



};

}



#endif  // XALANDUMMYIDENTWRITER_HEADER_GUARD_1357924680
